# Minimalism

*No gain, no pain.*

In context of [technology](tech.md) minimalism is a design philosophy which puts great emphasis on [simplicity](kiss.md), it says technology should be as simple as possible while still achieving given goal, possibly even a little bit simpler. Minimalism is one of the most (if not the most) important concepts in [programming](programming.md) and technology in general, it could almost be said that becoming a true expert in technology is strongly connected to realizing the importance of simplicity (see e.g. [Unix philosophy](unix_philosophy.md)). One of the first things to stress about minimalism is that it's firstly about **internal simplicity**, i.e. the simplicity of design/repairing/[hacking](hacking.md), and only secondly about the simplicity from the [user](user.md)'s point of view (otherwise we are only dealing with [pseudominimalism](pseudominimalism.md)). The opposite of minimalism is [maximalism](maximalism.md). See also [minimal viable program](minimal_viable_program.md).

Antoine de Saint-Exupéry sums it up with a quote: *we achieve perfection not when there is nothing more to add but when there is nothing left to take away.*

Minimalism is also an immensely important concept in [art](art.md), for example in architecture and design, and in addition there also exists the generalized concept of **life minimalism** which applies said philosophy to all areas of [life](life.md) and which numerous technological minimalists quite naturally start to follow along the way -- life minimalism is about letting go of objects, thoughts and desires that aren't necessarily needed because such things enslave us and mostly just make us more miserable; from time to time you should meditate a little bit about what it is that you really want and need and only keep that. Indeed this is nothing new under the Sun, this wisdom has been present for as long as humans have existed, most religions and philosophers saw a great value in [asceticism](asceticism.md), frugality and even poverty, as owning little leads to [freedom](freedom.md). For instance owning a [car](car.md) is kind of a slavery, you have to clean it, protect it, repair it, [maintain](maintenance.md) it, pay for parking space, pay for gas, pay for insurance -- this is not a small commitment and you sacrifice a significant part of your life and [head space](head_space.md) to it (especially considering additional commitments of similar magnitude towards your your house, garden, clothes, electronics, furniture, pets, bank accounts, social networks and so forth), a minimalist will rather choose to get a simple [suckless](suckless.md) bicycle, travel by public transport or simply walk.

**Minimalism is necessary for [freedom](freedom.md)** as a free technology can only be that over which no one has a [monopoly](bloat_monopoly.md), i.e. which many people and small parties can utilize, study and modify with affordable effort, without needing armies of technicians just for the maintenance of such technology. Minimalism goes against the creeping overcomplexity of technology which always brings huge costs and dangers, e.g. the cost of [maintenance](maintenance.md) and further development, obscurity, inefficiency ("[bloat](bloat.md)", wasting resources), consumerism, the increased risk of bugs, errors and failure.

There is a so called *[airplane rule](airplane_rule.md)* that states a plane with two engines has twice as many engine problems than a plane with a single engine.

Up until recently in history every engineer would tell you that *the better machine is that with fewer moving parts*. This still seems to hold e.g. in [mathematics](math.md), a field not yet so spoiled by huge commercialization and mostly inhabited by the smartest people -- there is a tendency to look for the most minimal equations -- such equations are considered [beautiful](beauty.md). Science also knows this rule as the [Occam's razor](occams_razor.md). In technology invaded by aggressive commercialization the situation is different, minimalism lives only in the underground and is ridiculed by the mainstream propaganda. Some the minimalist movements, terms and concepts at least somewhat connected to minimalism include (watch out for SJWs, [pseudominimalism](pseudominimalism.md), Nazis etc., we don't automatically fully embrace all things on this list):

- [suckless](suckless.md)
- [cat-v](cat_v.md)
- [bitreich](bitreich.md)
- [Collapse OS](collapseos.md)/[Dusk OS](duskos.md), [collapse](collapse.md) computing
- [less retarded software](lrs.md)
- [Unix philosophy](unix_philosophy.md)
- [KISS](kiss.md)
- [countercomplex](countercomplex.md)
- [permacomputing](permacomputing_wiki.md) (SJW cancer warning)
- [less is more](less_is_more.md)/[worse is better](worse_is_better.md)
- [appropriate technology](appropriate_tech.md)
- [neoluddism](neoluddism.md)
- [reactionary software](reactionary_software.md) (bordering with [pseudominimalism](pseudominimalism.md))
- [plan9](plan9.md), [openbsd](openbsd.md), KISS GNU/Linux and similar (however often obsessed with [bloatcryption](encryption.md), may contain [pseudominimalism](pseudominimalism.md))
- [100rabbits](100rabbit.md) (beware of [SJW](sjw.md) poison)
- [small Internet](small_internet.md), web 1.0, web 0.5, [gopher](gopher.md), [gemini](gemini.md) (watch out: gemini is SJW pseudominimalist bloatcryption poison), ...
- [primitivism](primitivism.md)/[anarcho primitivism](anprim.md), [low tech](low_tech.md), ...
- for potential weaker links to minimalism also check out [retro](retro.md)/[old](old.md)/[boomer](boomer.md) tech, [salvage computing](salvage_computing.md), [degrowth](degrowth.md), [Amish](amish.md), [technophobia](technophobia.md), [demoscene](demoscene.md), [code golf](golf.md), [lightweight](lightweight.md) software, [fantasy consoles](fantasy_console.md) (sadly mostly pseudominimalism), communities around [plain text](plain_text.md), [pubnixes](pubnix.md), some GNU/Linux distros (e.g. [Arch](arch.md), [Gentoo](gentoo.md), KISS Linux, ...), [IRC](irc.md) communities and so on.
- ...

Under [capitalism](capitalism.md) technological minimalism is suppressed in the mainstream as it goes against [corporate](corporation.md) interests, i.e. those of having monopoly control over technology, even if such technology is "[FOSS](foss.md)" (which then becomes just a cool brand, see [openwashing](openwashing.md)). We may, at best, encounter a "shallow" kind of minimalism, so called [pseudominimalism](pseudominimalism.md) which only tries to make things appear minimal, e.g. aesthetically, and hides ugly overcomplicated internals under the facade. [Apple](apple.md) is infamous for this [shit](shit.md).

There are movements such as [appropriate technology](appropriate_tech.md) (described by E. F. Schumacher in a work named *Small Is Beautiful: A Study of Economics As If People Mattered*) advocating for small, efficient, decentralized technology, because that is what best helps people. 

**Does minimalism mean we have to give up the nice things?** Well, not really, it is more about giving up the [bullshit](bullshit.md), getting rid of addiction and changing an attitude. People addicted to [modern](modern.md) consumerist technology often worry that with minimalism they will lose their drug, typically [games](game.md) or something similar. Remember that with minimalism **we can still have technology for entertainment**, just a non-consumerist one -- instead of consuming a new game each month we may rather focus on creating deeper games that may last longer, e.g. those of a [easy to learn, hard to master](easy_to_learn_hard_to_master.md) kind and building communities around them, or on modifying existing games rather than creating new ones from scratch over and over. Sure, technology would LOOK different, our computer interfaces may become less of a thing of fashion, our games may rely more on aesthetics than realism, but ultimately minimalism can be seen just as trying to achieve the same effect while minimizing waste. If you've been made addicted to bullshit such as buying a new GPU each month so that you can run games at 1000 FPS at progressively higher resolution then of course yes, you will have to suffer a bit of a withdrawal just as a heroin addict suffers when quitting the drug, but just as him in the end you'll be glad you did it.

## Importance Of Minimalism: Simplicity Brings Freedom

It can't be stressed enough that minimalism is absolutely required for technological freedom, i.e. people having, in **practical** ways, control over their tools. While in today's society it is important to have legal freedoms, i.e. support [free software](free_software.md), we must not forget that this isn't enough, a freedom on paper means nothing if it can't be practiced. We need both legal AND [de facto](de_facto.md) freedom over technology, the former being guaranteed by a free [license](license.md), the latter by minimalism. Minimal, simple technology will increase the pool of people and parties who may practice the legal freedoms -- i.e. those to use, study, modify and share -- and therefore ensure that the technology will be developed according to what people need, NOT according to what a corporation needs (which is usually the opposite).

Even if a user of software is not a programmer himself, it is important he chooses to use minimal tools because that makes it more likely his tools can be repaired or improved by SOMEONE from the people. Some people naively think that if they're not programmers, it doesn't matter if they have access and rights to the program's source code, but indeed that is not the case. You want to choose tools that can easily be analyzed and repaired by someone, even if you yourself can't do it.

Minimalism and simplicity increases freedom even of [proprietary](proprietary.md) technology which can be seen e.g. on [games](game.md) for old systems such as [GameBoy](gameboy.md) or [DOS](dos.md) -- these games, despite being proprietary, can and are easily and plentifully played, modified and shared by the people, DESPITE not being free legally, simply because it is easy to handle them due to their simplicity. This just further confirms the correlation of freedom and minimalism.

## See Also

- [primitivism](primitivism.md)
- [single instruction computer](single_instruction.md)